#! /usr/bin/env python

"""Extract the Magma code from a literate Magma script

  $Id: MagmaCode.py,v 1.6 2005/07/02 06:20:38 don Exp $

	MagmaCode.py
	D E Taylor
	21 July 2002

 A literate Magma script consists of narrative interspersed with
 executable Magma code.  Each code line begins with > 
 This program extracts the lines beginning with > and then removes the
 prompt (>) and one following blank (if it exists).

 Lines beginning with %% at the beginning of the file are turned into
 Magma comments by replacing the %% with // and then copied to the output

 A line beginning with %%stop halts the output of Magma code. It is resumed
 by a line beginning with %%start
 
 The first version of this program assumed that every initial > was
 followed by a space and therefore simply stripped the first two characters
 from each code line.
 
        Command line options:

          -d  debug

        Requires:
          Python 2.0
"""
import sys, getopt, time

VERSION = "MagmaCode 1.2"
DEBUG = 0

def main():
  global DEBUG
  def fail(msg = None):
    if msg:
      print msg
    print 'Usage: %s [-d] <script>' % sys.argv[0]
    sys.exit(1)

  if len(sys.argv) < 2:
    fail()

  try:
    opts, files = getopt.getopt(sys.argv[1:],'d')
  except getopt.GetoptError,e:
    print >> sys.stderr, e
    sys.exit(1)
    
  if not files:
    fail()
  for opt in opts:
    if opt[0] == '-d':
      DEBUG = 1

  infile = files[0]
  path = infile.split('.')
  if len(path) > 1:
    ext = path.pop()
  else:
    ext = ''

  if ext == "m" or ext == "M":
    fail("A literate script should not have a .m extension")

  path.append('m')
  outfile = '.'.join(path)

  if DEBUG:
    print >> sys.stderr, "[In]  = "+infile
    print >> sys.stderr, "[Out] = "+outfile

  try:
    f = open(infile)
    out = open(outfile,'w')
  except IOError,e:
    print >> sys.stderr, e
    sys.exit(1)

  out.write('// Converted from '+
    infile+' by '+VERSION+' on '+
    time.asctime(time.localtime(time.time()))+'\n//\n')

  for line in f:
    line = line.lstrip()
    if line.startswith('%%'):
      out.write('//'+line[2:])
    else:
      break

  out.write('\n')

  incode = 0
  active = True
  for line in f:
    line = line.lstrip()
    if line.startswith('\\end{document}'):
      break
    elif line.startswith('%%stop'):
      active = False
    elif line.startswith('%%start'):
      active = True
    elif line.startswith('>'):
      incode = 1
      line = line[1:]
      if line.startswith(' '):
        line = line[1:]
      if active:
        out.write(line)
    elif incode:
      if active:
        out.write('\n')
      incode = 0

  f.close()
  out.close()

if __name__ == '__main__':
  main()

